import * as vscode from "vscode";
import { join } from "path";
import sirv from "sirv";
import getPort from "get-port";
import Viewer from "./lib/viewer";
import runInVm from "./lib/runInVm";
import save from "./lib/save";
import { View } from "./lib/sandbox";

export function activate(context: vscode.ExtensionContext) {
  let viewer: Viewer | null = null;

  context.subscriptions.push(
    vscode.commands.registerCommand(
      "composition-environment.view",
      async () => {
        try {
          const content = runInVm();
          if (content instanceof View) {
            if (!viewer) {
              const assets = sirv(join(context.extensionPath, "dist/viewer"));
              const port = await getPort({ port: 3000 });
              viewer = new Viewer(assets, port);
            }
            viewer.run(content.data);
          } else {
            vscode.window.showErrorMessage("No View returned");
          }
        } catch (e: any) {
          vscode.window.showErrorMessage(e.message);
        }
      }
    )
  );

  context.subscriptions.push(
    vscode.commands.registerCommand("composition-environment.save", () => {
      try {
        const content = runInVm();
        if (content instanceof View) {
          save(content.data);
        } else {
          vscode.window.showErrorMessage("No View returned");
        }
      } catch (e: any) {
        vscode.window.showErrorMessage(e.message);
      }
    })
  );
}

export function deactivate() {}
