import * as vscode from "vscode";
import { writeFile } from "fs";
import toSvg from "./toSvg";

async function save(score: any) {
  const editor = vscode.window.activeTextEditor;
  if (!editor) {
    throw Error(`No text editor`);
  }

  const format = await vscode.window.showQuickPick([
    "MusicXML",
    "MEI",
    "SVG",
    "Timing Data",
  ]);

  if (!format) return;

  const folders = vscode.workspace.workspaceFolders;
  const defaultUri = folders ? folders[0].uri : undefined;

  let data: any;
  let filters: any;

  switch (format) {
    case "MusicXML":
      data = score.output("musicXml");
      filters = { xml: ["xml"] };
      break;
    case "MEI":
      data = score.output("mei");
      filters = { mei: ["mei"] };
      break;
    case "SVG":
      data = toSvg(score.output("mei"));
      filters = { svg: ["svg"] };
      break;
    case "Timing Data":
      data = score.output("timingData");
      filters = { json: ["json"] };
  }

  const uri = await vscode.window.showSaveDialog({ defaultUri, filters });

  if (!uri) return;

  await writeFile(uri.path, data, (err) => {
    if (err) {
      throw err;
    }
  });
}

export default save;
