import * as vscode from "vscode";
import ip from "ip";
import startServer from "./startServer";
import WebSocket from "ws";
import toSvg from "./toSvg";

class Viewer {
  private _ip = ip.address();
  private _port: number;
  private _wss: WebSocket.Server;
  private _data: any = null;

  constructor(assets: any, port: number) {
    this._port = port;
    const server = startServer(assets, this._port);
    this._wss = new WebSocket.Server({ server });
    this._wss.on("connection", (ws) => {
      if (this._data) {
        ws.send(this._data);
      }
    });
    vscode.window.showInformationMessage(
      `Score viewer available at http://${this._ip}:${this._port}`
    );
  }

  private _broadcast() {
    if (this._data) {
      this._wss.clients.forEach((client) => {
        if (client.readyState === WebSocket.OPEN) {
          client.send(this._data);
        }
      });
    }
  }

  run(score: any) {
    const mei = score.output("mei");
    const data = score.output("eventData");
    const svg = toSvg(mei);
    this._data = JSON.stringify({ svg, data });
    if (!this._wss.clients.size) {
      vscode.env.openExternal(
        vscode.Uri.parse(`http://localhost:${this._port}`)
      );
    }
    this._broadcast();
  }
}

export default Viewer;
