import Fraction from "fraction.js";
import Fragment from "./fragment";
import Direction from "./direction";
import setDuration from "./util/setDuration";
import { formatBeatValue, formatTempo } from "./util/formatTempo";
import { AccelProps, DurationInput } from "./types";

class AccelFragment {
  fragment: Fragment;
  props: AccelProps;
  direction: Direction;

  constructor(
    fragment: Fragment,
    startTempo: Fraction,
    endTempo: Fraction,
    numBeats: number,
    beatValue: DurationInput
  ) {
    let { duration } = fragment;
    const { type, dots, beats } = setDuration(beatValue);
    const accelDuration = beats.mul(numBeats);
    if (duration.compare(accelDuration) > 0) {
      throw Error(`Fragment is longer than accel`);
    }
    this.fragment = fragment;
    this.props = { startTempo, endTempo, numBeats, type, dots, beats };
    this.direction = new Direction(
      `${formatBeatValue(type, dots)} ${formatTempo(
        startTempo
      )} \u2192 ${formatTempo(endTempo)}`
    );
  }
}

export default AccelFragment;
