import Event from "./event";
import Layer from "./layer";

function createLayer(input: string) {
  const content = input.matchAll(
    /(?<chordStart>[<]?)(?<pitch>[a-g|r]?[sf]*[',]*)(?<chordEnd>[>]?)(?<dur>[0-9]*[.]*)(?<tie>[~]?)/g
  );
  const events: Event[] = [];
  let dur = "4";
  let pitches = ["c"];
  let chord = false;
  let currentPitches: string[] = [];
  for (const match of content) {
    if (!match.groups) {
      throw Error(`No capturing groups`);
    }
    let {
      chordStart,
      pitch: currentPitch,
      chordEnd,
      dur: currentDur,
      tie,
    } = match.groups;
    const rest = currentPitch === "r";
    if (currentPitch || currentDur) {
      if (chordStart || chord) {
        if (chordEnd) {
          chord = false;
        } else {
          if (rest) {
            throw Error(`<chord> should not contain rests`);
          }
          if (currentDur) {
            throw Error(`<chord> should not contain durations`);
          }
          if (tie) {
            throw Error(`<chord> should not contain ties`);
          }
          chord = true;
          currentPitches.push(currentPitch);
        }
      }
      if (!chord) {
        currentPitches.push(currentPitch);
        if (currentDur) {
          dur = currentDur;
        }
        if (currentPitch) {
          pitches = currentPitches;
        }
        events.push(
          new Event().setDuration(dur).setPitches(pitches).setTie(!!tie)
        );
        currentPitches = [];
      }
    }
  }
  return new Layer(events);
}

export default createLayer;
