import Fraction from "fraction.js";
import Duration from "./duration";
import Event from "./event";
import Layer from "./layer";
import { Entry } from "./types";

abstract class Entries extends Duration {
  children!: Entry[];
  props: any;

  constructor(values: Entry[] = [], props: any = {}) {
    const children: Entry[] = [];
    let duration = new Fraction(0);
    let lcd = new Fraction(1);

    for (const value of values) {
      if (value instanceof Layer) {
        children.push(...value.children);
      } else {
        children.push(value);
      }
      duration = duration.add(value.duration);
      lcd = lcd.lcm(value.lcd);
    }

    super(children, duration, lcd);
    this.props = props;
  }

  pipe(...transforms: Array<(values: Entry[]) => Iterable<Entry>>) {
    const children = transforms.reduce(
      (acc, cur) => [...cur(acc)],
      [...this.children]
    );
    return this._copy(children);
  }

  pipeEvents(...transforms: Array<(event: Event) => Event>) {
    return this.pipe(
      ...transforms.map(
        (transform) =>
          (values: Entry[]): Entry[] =>
            values.map((value) =>
              value instanceof Event
                ? transform(value)
                : value.pipeEvents(transform)
            )
      )
    );
  }

  protected _copy(children: Entry[], change: any = {}) {
    return new (this.constructor as any)(children, {
      ...this.props,
      ...change,
    });
  }
}

export default Entries;
