import Fraction from "fraction.js";
import x from "xastscript";
import Layers from "./layers";
import Layer from "./layer";
import {
  DurationInput,
  PipeProps,
  MusicXmlProps,
  MeiProps,
  EventDataProps,
} from "./types";

class Staff extends Layers {
  children!: Layer[];

  constructor(children: Layer[] = []) {
    super(children);
  }

  add(value: Layer | Staff) {
    const children = [...this.children];
    if (value instanceof Layer) {
      if (!children.length) {
        children.push(value);
      } else {
        children[0] = children[0].add(value);
      }
    } else if (value instanceof Staff) {
      value.children.forEach((layer, i) => {
        if (i === children.length) {
          children.push(new Layer().addRests(this.duration));
        }
        children[i] = children[i].add(layer);
      });
    } else {
      throw Error(`${value} is not a Staff or Layer`);
    }
    return new Staff(children);
  }

  merge(value: Staff) {
    const children = [...this.children, ...value.children];
    return new Staff(children);
  }

  tuplet(
    outer: number,
    inner: number,
    beatValue?: DurationInput,
    hidden?: boolean
  ) {
    const children = this.children.map((layer) =>
      layer.tuplet(outer, inner, beatValue, hidden)
    );
    return new Staff(children);
  }

  pipe(
    transform: (layer: Layer, props: PipeProps) => Layer,
    props: PipeProps
  ) {
    const children = this.children.map((layer, i) =>
      transform(layer, { ...props, layerNum: i })
    );
    return new Staff(children);
  }

  toMusicXml(props: MusicXmlProps) {
    return this.children.map((layer, i) => {
      props = { ...props, voice: props.voice + i };
      const output = layer.toMusicXml(props);
      if (i !== this.children.length - 1) {
        output.push(
          x("backup", [
            x("duration", layer.duration.mul(4).mul(props.lcd).valueOf()),
          ])
        );
      }
      return output;
    });
  }

  toMei(props: MeiProps) {
    return this.children.map((layer, i) => {
      return x("layer", { n: i + 1 }, layer.toMei(props));
    });
  }

  toEventData(props: EventDataProps) {
    return this.children.map((layer) => layer.toEventData(props));
  }

  toBeatData(lcd: Fraction) {
    return [...this.children[0].toBeatData(lcd)];
  }

  toTimingData() {
    return this.children[0].toTimingData();
  }
}

export default Staff;
