import Fraction from "fraction.js";
import Entries from "./entries";
import Event from "./event";
import {
  Entry,
  TempoProps,
  MusicXmlProps,
  MeiProps,
  EventDataProps,
} from "./types";

class Tempo extends Entries {
  children!: Entry[];
  props!: TempoProps;

  constructor(values: Entry[], props: TempoProps) {
    super(values, props);
    const { tempo } = props;
    this.duration = this.duration.mul(tempo.inverse());
    this.lcd = this.lcd.mul(tempo);
  }

  toMusicXml(props: MusicXmlProps): any {
    const { tempo } = this.props;
    const timeMod = props.timeMod.mul(tempo);
    return this.children.map((entry) =>
      entry.toMusicXml({ ...props, timeMod })
    );
  }

  toMei(props: MeiProps): any {
    const { tempo } = this.props;
    return this.children.map((entry) => entry.toMei({ ...props, tempo }));
  }

  toEventData(props: EventDataProps): any {
    const { tempo } = this.props;
    const timeMod = props.timeMod.mul(tempo);
    props = { ...props, timeMod };
    const events: any = [];
    this.children.forEach((entry) =>
      entry instanceof Event
        ? events.push(entry.toEventData(props))
        : events.push(...entry.toEventData(props))
    );
    return events;
  }

  *toBeatData(lcd: Fraction, offset: Fraction) {
    let elapsed = offset;
    const { tempo, beats } = this.props;
    while (!elapsed.equals(this.duration.add(offset))) {
      elapsed = elapsed.add(beats.mul(tempo.inverse()));
      yield elapsed.mul(4).mul(lcd).valueOf();
    }
  }
}

export default Tempo;
