import Fraction from "fraction.js";
import Fragment from "./fragment";
import Direction from "./direction";
import setDuration from "./util/setDuration";
import { formatBeatValue, formatTempo } from "./util/formatTempo";
import { TempoProps, DurationInput } from "./types";

class TempoFragment {
  fragment: Fragment;
  props: TempoProps;
  direction: Direction;

  constructor(fragment: Fragment, tempo: Fraction, beatValue: DurationInput) {
    let { duration } = fragment;
    const { value, type, dots, beats } = setDuration(beatValue);
    if (duration.mod(beats).compare(0)) {
      throw Error(`Tempo ${tempo.n}:${tempo.d} must contain '${value}' beats`);
    }
    this.fragment = fragment;
    this.props = { tempo, type, dots, beats };
    this.direction = new Direction(
      `${formatBeatValue(type, dots)} ${formatTempo(tempo)}`
    );
  }
}

export default TempoFragment;
