import x from "xastscript";
import Entries from "./entries";
import Event from "./event";
import {
  Entry,
  TupletProps,
  MusicXmlProps,
  MeiProps,
  EventDataProps,
} from "./types";

class Tuplet extends Entries {
  children!: Entry[];
  props!: TupletProps;

  constructor(values: Entry[], props: TupletProps) {
    super(values, props);

    const { outer, inner, value, beats } = props;
    if (this.duration.compare(beats.mul(outer))) {
      throw Error(
        `Tuplet ${outer}:${inner} must contain ${outer} '${value}' beats`
      );
    }

    this.duration = beats.mul(inner);
    this.lcd = this.lcd.mul([outer, inner]);
  }

  toMusicXml(props: MusicXmlProps): any {
    const { outer, inner } = this.props;
    const timeMod = props.timeMod.mul(outer, inner);
    let { number, start, stop } = props.tuplets;
    number++;
    start = [...start, { number, ...this.props }];
    stop = [...stop, number];
    return this.children.map((entry, i) => {
      const tuplets = {
        number,
        start: i === 0 ? start : [],
        stop: i === this.children.length - 1 ? stop : [],
      };
      return entry.toMusicXml({ ...props, timeMod, tuplets });
    });
  }

  toMei(props: MeiProps): any {
    const { outer, inner, type, dots } = this.props;
    return x(
      "tuplet",
      { dur: type, dots, num: outer, numbase: inner },
      this.children.map((entry) => entry.toMei(props))
    );
  }

  toEventData(props: EventDataProps): any {
    const { outer, inner } = this.props;
    const timeMod = props.timeMod.mul(outer, inner);
    props = { ...props, timeMod };
    const events: any = [];
    this.children.forEach((entry) =>
      entry instanceof Event
        ? events.push(entry.toEventData(props))
        : events.push(...entry.toEventData(props))
    );
    return events;
  }
}

export default Tuplet;
