import Fraction from "fraction.js";
import Event from "./event";
import Layer from "./layer";
import Tuplet from "./tuplet";
import Tempo from "./tempo";
import Direction from "./direction";
import { BaseTempo } from "score-timing-data";

export interface Tie {
  stop: boolean;
  start: boolean;
}

export interface PitchProps {
  pitch: string;
  pitchClass: number;
  pname: string;
  accid: string;
  oct: number;
  tie: Tie;
}

export interface Duration {
  value: string;
  type: number;
  dots: number;
  beats: Fraction;
}

export interface EventProps {
  duration: Duration;
  type: "space" | "rest" | "note" | "chord";
  tie: boolean;
  directions: Direction[];
  id: string;
}

export type PitchInput = string | number;

export type DurationInput = string | number;

export interface EventInput {
  pitches?: PitchInput | PitchInput[];
  duration?:
    | DurationInput
    | [number, number]
    | { n: number; d: number }
    | Fraction;
  tie?: boolean;
  hidden?: boolean;
}

export interface TupletProps {
  outer: number;
  inner: number;
  value: string;
  type: number;
  dots: number;
  beats: Fraction;
  hidden: boolean;
}

export interface TempoProps {
  tempo: Fraction;
  type: number;
  dots: number;
  beats: Fraction;
}

export interface AccelProps {
  startTempo: Fraction;
  endTempo: Fraction;
  numBeats: number;
  type: number;
  dots: number;
  beats: Fraction;
}

export type Entry = Layer | Tuplet | Tempo | Event;

export interface PartProps {
  name: string;
  clefs: string[];
}

export interface PipeProps {
  staffNum: number;
  layerNum: number;
}

export interface MusicXmlTuplet extends TupletProps {
  number: number;
}

export interface MusicXmlProps {
  lcd: Fraction;
  timeMod: Fraction;
  staff: number;
  voice: number;
  tuplets: {
    number: number;
    start: MusicXmlTuplet[];
    stop: number[];
  };
}

export interface MeiDirections {
  id: string;
  directions: Direction[];
}

export interface MeiProps {
  staffNum: number;
  directions: MeiDirections[];
  tempo?: Fraction;
}

export interface EventDataProps {
  lcd: Fraction;
  timeMod: Fraction;
}

export interface ScoreProps {
  baseTempo: BaseTempo;
}

export type OutputFormats =
  | "musicXml"
  | "mei"
  | "eventData"
  | "beatData"
  | "timingData";
