import Fraction from "fraction.js";

export function notPowerOfTwo(value: number) {
  return Math.log2(value) % 1;
}

function setBeats(type: number, dots: number) {
  const base = new Fraction(type).inverse();
  const multiplier = (Math.pow(2, dots + 1) - 1) / Math.pow(2, dots);
  return base.mul(multiplier);
}

function setDuration(value: string | number = 1 / 4) {
  let type: number;
  let dots = 0;
  let beats: Fraction;
  if (typeof value === "string") {
    const match = value.match(/(?<type>[0-9]*)(?<dots>[.]*)/);
    if (!match?.groups) {
      throw Error(`Duration '${value}' is not valid`);
    }
    const { groups } = match;
    type = parseInt(groups.type);
    if (isNaN(type)) {
      throw Error(`Duration '${value}' is not valid`);
    }
    dots = groups.dots.length;
    beats = setBeats(type, dots);
  } else if (typeof value === "number") {
    beats = new Fraction(value);
    type = 1 / value;
    value = type.toString();
  } else {
    throw Error(`Duration '${value}' must be string or number`);
  }
  if (notPowerOfTwo(type)) {
    throw Error(`Duration '${value}' must be a power of two`);
  }
  return {
    value,
    type,
    dots,
    beats,
  };
}

export default setDuration;
