import { LitElement, css, html } from "lit";
import { customElement, property } from "lit/decorators.js";
import { connect } from "pwa-helpers/connect-mixin";
import { unsafeHTML } from "lit-html/directives/unsafe-html.js";
import store from "../store";
import { setData } from "../store/reducer";
import "./header";
import "./content";
import "./footer";
import "../midi";

@customElement("score-viewer")
class Viewer extends connect(store)(LitElement) {
  @property()
  svg: any;

  static get styles() {
    return css`
      * {
        --mdc-theme-primary: black;
        --mdc-theme-secondary: black;
      }
      score-header {
        height: 15%;
        display: flex;
        justify-content: space-around;
        align-items: center;
      }
      score-content {
        height: 75%;
        display: flex;
      }
      score-footer {
        height: 10%;
        display: flex;
        justify-content: space-around;
        align-items: center;
      }
      .current {
        fill: #e68a00;
      }
    `;
  }

  connectedCallback() {
    super.connectedCallback();

    const setScore = (content: any) => {
      const { svg, data } = content;
      this.svg = svg;
      store.dispatch(setData(data));
    };

    (window as any).setScore = setScore;

    const socket = new WebSocket(`ws://${window.location.host}`);

    socket.onmessage = ({ data }) => setScore(JSON.parse(data));
  }

  render() {
    return html`
      <score-header></score-header>
      <score-content> ${this.svg ? unsafeHTML(this.svg) : null} </score-content>
      <score-footer></score-footer>
    `;
  }
}

export default Viewer;
