import WebMidi, { Output } from "webmidi";
import store, { RootState } from "../store";
import { setMidiEnabled } from "../store/reducer";
import { MidiEvent } from "../store/types";

const getMidiOutput = (state: RootState) => state.midiOutput;
const getCurrentMidiEvents = (state: RootState) => state.currentMidiEvents;
const getStartTime = (state: RootState) => state.startTime;
const getStartOffset = (state: RootState) => state.startOffset;

let previousOutputName = "";
let previousMidiEvents: MidiEvent[][][];
let output: Output;

function setOutput(state: RootState) {
  const currentOutputName = getMidiOutput(state);
  if (currentOutputName !== previousOutputName) {
    output = WebMidi.getOutputByName(currentOutputName) as Output;
    previousOutputName = currentOutputName;
  }
}

function scheduleEvents(state: RootState) {
  const currentMidiEvents = getCurrentMidiEvents(state);
  if (currentMidiEvents !== previousMidiEvents) {
    if (output) {
      const startTime = getStartTime(state);
      const startOffset = getStartOffset(state);
      currentMidiEvents.forEach((part, i) => {
        const channel = i + 1;
        part.forEach((sequence) => {
          sequence.forEach(({ start, pitches }) => {
            const time = start + startTime - startOffset;
            pitches.forEach(({ duration, pitchNum }) => {
              output.playNote(pitchNum, channel, { time, duration });
            });
          });
        });
      });
    }
    previousMidiEvents = currentMidiEvents;
  }
}

WebMidi.enable((error) =>
  error ? console.log(error) : store.dispatch(setMidiEnabled(true))
);

store.subscribe(() => {
  const state = store.getState();
  setOutput(state);
  if (output) {
    scheduleEvents(state);
  }
});
