import { createSlice, PayloadAction } from "@reduxjs/toolkit";
import setEvents from "./util/setEvents";
import setEventIndexes from "./util/setEventIndexes";
import setStop from "./util/setStop";
import setCurrentMidiEvents from "./util/setCurrentMidiEvents";
import setCurrentScoreEvents from "./util/setCurrentScoreEvents";
import setStartOffset from "./util/setStartOffset";
import { RootState, Data } from "./types";

const initialState: RootState = {
  midiEnabled: false,
  midiOutput: "",
  baseTempo: 60,
  midiBufferSize: 1000,
  data: null,
  scoreEvents: [[[]]],
  scoreEventIndexes: [[]],
  midiEvents: [[[]]],
  midiEventIndexes: [[]],
  currentMidiEvents: [[[]]],
  totalTime: 0,
  startTime: 0,
  startOffset: 0,
  syncing: false,
  playing: false,
  scrollPos: 0,
};

const root = createSlice({
  name: "root",
  initialState,
  reducers: {
    setData(state, action: PayloadAction<Data>) {
      state.data = action.payload;
      setEvents(state);
      setEventIndexes(state);
    },
    setMidiEnabled(state, action: PayloadAction<boolean>) {
      state.midiEnabled = action.payload;
    },
    setMidiOutput(state, action: PayloadAction<string>) {
      state.midiOutput = action.payload;
    },
    setBaseTempo(state, action: PayloadAction<string>) {
      state.baseTempo = parseInt(action.payload);
      setEvents(state);
      setStartOffset(state);
    },
    setCurrentTime(state, action: PayloadAction<number>) {
      let currentTime = action.payload;
      setCurrentMidiEvents(state, currentTime);
      setCurrentScoreEvents(state, currentTime);
    },
    start(
      state,
      action: PayloadAction<{ startTime: number; startOffset: number }>
    ) {
      if (state.data) {
        const { startTime, startOffset } = action.payload;
        state.startTime = startTime;
        state.startOffset = startOffset;
        state.syncing = true;
      }
    },
    stop(state) {
      setStop(state);
    },
    setScrollPos(state, action: PayloadAction<number>) {
      if (state.data) {
        // MOVED DUE TO LAYOUT THRASHING
        state.scrollPos = action.payload;
        if (!state.syncing && !state.playing) {
          // state.scrollPos = action.payload
          setStartOffset(state);
          setEventIndexes(state);
        }
      }
    },
  },
});

const { actions, reducer } = root;

export const {
  setData,
  setMidiEnabled,
  setMidiOutput,
  setBaseTempo,
  setCurrentTime,
  start,
  stop,
  setScrollPos,
} = actions;

export default reducer;
