import { RootState, MidiEvent } from "../types";

function setCurrentMidiEvents(state: RootState, currentTime: number) {
  if (state.midiEnabled && (state.syncing || state.playing)) {
    const midiTime =
      currentTime - state.startTime + state.startOffset + state.midiBufferSize;
    let update = false;
    const currentMidiEvents = state.midiEvents.map((part, partIndex) =>
      part.map((sequence, sequenceIndex) => {
        const events: MidiEvent[] = [];
        for (
          let i = state.midiEventIndexes[partIndex][sequenceIndex];
          i <= sequence.length;
          i++
        ) {
          const event = sequence[i];
          if (event && event.start < midiTime) {
            update = true;
            events.push(event);
          } else {
            state.midiEventIndexes[partIndex][sequenceIndex] = i;
            break;
          }
        }
        return events;
      })
    );
    if (update) {
      state.currentMidiEvents = currentMidiEvents;
    }
  }
}

export default setCurrentMidiEvents;
