import { RootState, ScoreEvent, MidiEvent } from "../types";

function setEvents(state: RootState) {
  const { data, baseTempo } = state;
  if (data) {
    const { divisions } = data;
    const division = (60 / baseTempo / divisions) * 1000;
    let totalTime = 0;
    const scoreEvents = data.events.map((part) =>
      part.map((sequence) => {
        let currentTime = 0;
        const output = sequence.map(({ id, duration, pitches }): ScoreEvent => {
          const start = currentTime;
          duration *= division;
          const end = start + duration;
          currentTime = end;
          return { id, start, duration, end, pitches };
        });
        if (currentTime > totalTime) {
          totalTime = currentTime;
        }
        return output;
      })
    );
    const midiEvents = scoreEvents.map((part) =>
      part.map((sequence) => {
        const events: { start: number; duration: number; pitchNum: number }[] =
          [];
        const tied: { [index: number]: { start: number; duration: number } } =
          {};
        sequence.forEach(({ start, duration, pitches }) => {
          pitches.forEach(({ num, tie }) => {
            if (tied[num]) {
              tied[num].duration += duration;
              if (!tie) {
                events.push({ ...tied[num], pitchNum: num });
                delete tied[num];
              }
            } else if (tie) {
              tied[num] = { start, duration };
            } else {
              events.push({ start, duration, pitchNum: num });
            }
          });
        });
        const combinedEvents: MidiEvent[] = [];
        let current: MidiEvent = { start: 0, pitches: [] };
        events.forEach(({ start, duration, pitchNum }) => {
          if (start === current.start) {
            current.pitches.push({ duration, pitchNum });
          } else {
            combinedEvents.push(current);
            current = { start, pitches: [{ duration, pitchNum }] };
          }
        });
        combinedEvents.push(current);
        return combinedEvents;
      })
    );
    state.scoreEvents = scoreEvents;
    state.midiEvents = midiEvents;
    state.totalTime = totalTime;
  }
}

export default setEvents;
